@@ -134,6 +134,8 @@ static gboolean gst_flv_mux_are_all_pads_eos (GstFlvMux * mux);
 static GstFlowReturn gst_flv_mux_update_src_caps (GstAggregator * aggregator,
     GstCaps * caps, GstCaps ** ret);
 static guint64 gst_flv_mux_query_upstream_duration (GstFlvMux * mux);
+static GstClockTime gst_flv_mux_segment_to_running_time (const GstSegment *
+    segment, GstClockTime t);
 
 static GstFlowReturn
 gst_flv_mux_pad_flush (GstAggregatorPad * pad, GstAggregator * aggregator)
@@ -147,6 +149,47 @@ gst_flv_mux_pad_flush (GstAggregatorPad * pad, GstAggregator * aggregator)
   return GST_FLOW_OK;
 }
 
+static gboolean
+gst_flv_mux_skip_buffer (GstAggregatorPad * apad, GstAggregator * aggregator,
+    GstBuffer * buffer)
+{
+  GstFlvMuxPad *fpad = GST_FLV_MUX_PAD_CAST (apad);
+  GstFlvMux *mux = GST_FLV_MUX_CAST (aggregator);
+  GstClockTime t;
+
+  if (!mux->skip_backwards_streams)
+    return FALSE;
+
+  if (fpad->drop_deltas) {
+    if (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DELTA_UNIT)) {
+      GST_INFO_OBJECT (fpad, "Waiting for keyframe, dropping %" GST_PTR_FORMAT,
+          buffer);
+      return TRUE;
+    } else {
+      /* drop-deltas is set and the buffer isn't delta, drop flag */
+      fpad->drop_deltas = FALSE;
+    }
+  }
+
+  if (GST_CLOCK_TIME_IS_VALID (GST_BUFFER_DTS_OR_PTS (buffer))) {
+    t = gst_flv_mux_segment_to_running_time (&apad->segment,
+        GST_BUFFER_DTS_OR_PTS (buffer));
+
+    if (t < (GST_MSECOND * mux->last_dts)) {
+      GST_WARNING_OBJECT (fpad,
+          "Timestamp %" GST_TIME_FORMAT " going backwards from last used %"
+          GST_TIME_FORMAT ", dropping %" GST_PTR_FORMAT,
+          GST_TIME_ARGS (t), GST_TIME_ARGS (GST_MSECOND * mux->last_dts),
+          buffer);
+      /* Look for non-delta buffer */
+      fpad->drop_deltas = TRUE;
+      return TRUE;
+    }
+  }
+
+  return FALSE;
+}
+
 static void
 gst_flv_mux_pad_class_init (GstFlvMuxPadClass * klass)
 {
@@ -156,6 +199,8 @@ gst_flv_mux_pad_class_init (GstFlvMuxPadClass * klass)
   gobject_class->finalize = gst_flv_mux_pad_finalize;
 
   aggregatorpad_class->flush = GST_DEBUG_FUNCPTR (gst_flv_mux_pad_flush);
+  aggregatorpad_class->skip_buffer =
+      GST_DEBUG_FUNCPTR (gst_flv_mux_skip_buffer);
 }
 
 static void
@@ -1172,10 +1217,25 @@ gst_flv_mux_buffer_to_tag_internal (GstFlvMux * mux, GstBuffer * buffer,
   if (GST_CLOCK_STIME_IS_VALID (pad->dts)) {
     pts = pad->pts / GST_MSECOND;
     dts = pad->dts / GST_MSECOND;
+    GST_LOG_OBJECT (mux,
+        "Pad %s: Created dts %" GST_TIME_FORMAT ", pts %" GST_TIME_FORMAT
+        " from rounding %" GST_TIME_FORMAT ", %" GST_TIME_FORMAT,
+        GST_PAD_NAME (pad), GST_TIME_ARGS (dts * GST_MSECOND),
+        GST_TIME_ARGS (pts * GST_MSECOND), GST_TIME_ARGS (pad->dts),
+        GST_TIME_ARGS (pad->pts));
   } else if (GST_CLOCK_TIME_IS_VALID (pad->last_timestamp)) {
     pts = dts = pad->last_timestamp / GST_MSECOND;
+    GST_DEBUG_OBJECT (mux,
+        "Pad %s: Created dts and pts %" GST_TIME_FORMAT
+        " from rounding last pad timestamp %" GST_TIME_FORMAT,
+        GST_PAD_NAME (pad), GST_TIME_ARGS (pts * GST_MSECOND),
+        GST_TIME_ARGS (pad->last_timestamp));
   } else {
     pts = dts = mux->last_dts;
+    GST_DEBUG_OBJECT (mux,
+        "Pad %s: Created dts and pts %" GST_TIME_FORMAT
+        " from last mux timestamp",
+        GST_PAD_NAME (pad), GST_TIME_ARGS (pts * GST_MSECOND));
   }
 
   /* We prevent backwards timestamps because they confuse librtmp,
@@ -1279,7 +1339,7 @@ gst_flv_mux_buffer_to_tag_internal (GstFlvMux * mux, GstBuffer * buffer,
     data[11] |= (pad->width << 1) & 0x02;
     data[11] |= (pad->channels << 0) & 0x01;
 
-    GST_DEBUG_OBJECT (mux, "Creating byte %02x with "
+    GST_LOG_OBJECT (mux, "Creating byte %02x with "
         "codec:%d, rate:%d, width:%d, channels:%d",
         data[11], pad->codec, pad->rate, pad->width, pad->channels);
 
@@ -1854,7 +1914,6 @@ gst_flv_mux_find_best_pad (GstAggregator * aggregator, GstClockTime * ts,
     gboolean timeout)
 {
   GstFlvMuxPad *best = NULL;
-  GstBuffer *buffer;
   GstClockTime best_ts = GST_CLOCK_TIME_NONE;
   GstIterator *pads;
   GValue padptr = { 0, };
@@ -1865,68 +1924,23 @@ gst_flv_mux_find_best_pad (GstAggregator * aggregator, GstClockTime * ts,
   while (!done) {
     switch (gst_iterator_next (pads, &padptr)) {
       case GST_ITERATOR_OK:{
-
-        GstFlvMux *mux = GST_FLV_MUX (aggregator);
         GstAggregatorPad *apad = g_value_get_object (&padptr);
-        GstFlvMuxPad *fpad = GST_FLV_MUX_PAD (apad);
-        gint64 t = GST_CLOCK_TIME_NONE;
+        GstClockTime t = GST_CLOCK_TIME_NONE;
+        GstBuffer *buffer;
 
         buffer = gst_aggregator_pad_peek_buffer (apad);
         if (!buffer) {
-          if (timeout || GST_PAD_IS_EOS (apad)) {
-            continue;
-          } else {
-            if (best)
-              gst_object_replace ((GstObject **) & best, NULL);
+          if (!timeout && !GST_PAD_IS_EOS (apad)) {
+            gst_object_replace ((GstObject **) & best, NULL);
             best_ts = GST_CLOCK_TIME_NONE;
-            break;
-          }
-        }
-
-        if (fpad->drop_deltas) {
-          if (mux->skip_backwards_streams
-              && GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DELTA_UNIT)) {
-            GST_INFO_OBJECT (apad,
-                "Dropped buffer %" GST_PTR_FORMAT " until keyframe", buffer);
-            gst_buffer_unref (buffer);
-            gst_aggregator_pad_drop_buffer (apad);
-            if (timeout) {
-              continue;
-            } else {
-              if (best)
-                gst_object_replace ((GstObject **) & best, NULL);
-              best_ts = GST_CLOCK_TIME_NONE;
-              break;
-            }
-          } else {
-            /* drop-deltas is set and the buffer isn't delta, drop flag */
-            fpad->drop_deltas = FALSE;
+            done = TRUE;
           }
+          break;
         }
 
         if (GST_CLOCK_TIME_IS_VALID (GST_BUFFER_DTS_OR_PTS (buffer))) {
           t = gst_flv_mux_segment_to_running_time (&apad->segment,
               GST_BUFFER_DTS_OR_PTS (buffer));
-
-          if (mux->skip_backwards_streams
-              && (t < (GST_MSECOND * mux->last_dts))) {
-            GST_WARNING_OBJECT (mux,
-                "Timestamp %" GST_TIME_FORMAT " going "
-                "backwards from last known %" GST_TIME_FORMAT ", dropping",
-                GST_TIME_ARGS (t), GST_TIME_ARGS (GST_MSECOND * mux->last_dts));
-            gst_buffer_unref (buffer);
-            gst_aggregator_pad_drop_buffer (apad);
-            /* Look for non-delta buffer */
-            fpad->drop_deltas = TRUE;
-            if (timeout) {
-              continue;
-            } else {
-              if (best)
-                gst_object_replace ((GstObject **) & best, NULL);
-              best_ts = GST_CLOCK_TIME_NONE;
-              break;
-            }
-          }
         }
 
         if (!GST_CLOCK_TIME_IS_VALID (best_ts) ||
@@ -1935,7 +1949,6 @@ gst_flv_mux_find_best_pad (GstAggregator * aggregator, GstClockTime * ts,
           best_ts = t;
         }
         gst_buffer_unref (buffer);
-        g_value_reset (&padptr);
         break;
       }
       case GST_ITERATOR_DONE:
@@ -1952,13 +1965,19 @@ gst_flv_mux_find_best_pad (GstAggregator * aggregator, GstClockTime * ts,
         g_assert_not_reached ();
         break;
     }
+    g_value_reset (&padptr);
   }
   g_value_unset (&padptr);
   gst_iterator_free (pads);
 
-  GST_DEBUG_OBJECT (aggregator,
-      "Best pad found with %" GST_TIME_FORMAT ": %" GST_PTR_FORMAT,
-      GST_TIME_ARGS (best_ts), best);
+  if (best) {
+    GST_DEBUG_OBJECT (aggregator,
+        "Best pad found with TS %" GST_TIME_FORMAT ": %" GST_PTR_FORMAT,
+        GST_TIME_ARGS (best_ts), best);
+  } else {
+    GST_DEBUG_OBJECT (aggregator, "Best pad not found");
+  }
+
   if (ts)
     *ts = best_ts;
   return best;
@@ -2007,6 +2026,15 @@ gst_flv_mux_aggregate (GstAggregator * aggregator, gboolean timeout)
     best = gst_flv_mux_find_best_pad (aggregator, &ts, timeout);
   }
 
+  if (best) {
+    buffer = gst_aggregator_pad_pop_buffer (GST_AGGREGATOR_PAD (best));
+    if (!buffer) {
+      /* We might have gotten a flush event after we picked the pad */
+      gst_object_unref (best);
+      return GST_AGGREGATOR_FLOW_NEED_DATA;
+    }
+  }
+
   if (mux->new_tags && mux->streamable) {
     GstBuffer *buf = gst_flv_mux_create_metadata (mux);
     if (buf)
@@ -2015,8 +2043,6 @@ gst_flv_mux_aggregate (GstAggregator * aggregator, gboolean timeout)
   }
 
   if (best) {
-    buffer = gst_aggregator_pad_pop_buffer (GST_AGGREGATOR_PAD (best));
-    g_assert (buffer);
     best->dts =
         gst_flv_mux_segment_to_running_time (&GST_AGGREGATOR_PAD
         (best)->segment, GST_BUFFER_DTS_OR_PTS (buffer));
